@@ -37,7 +37,6 @@ class Database:
                 ohlc_result = json.loads(json.loads(response.content).replace("'", "\"").replace('False','0').replace('True','1'))
             else:
 
-                #result = ast.literal_eval(json.loads(response.content))
                 result = json.loads(json.loads(response.content).replace("'", "\"").replace('[(', '\"[(').replace(')]',')]\"').replace('False','0').replace('True','1'))
                 ohlc_result = result[0]
 
@@ -80,6 +79,7 @@ class Database:
 
 
             if price_adj == True:
+
                 qfq_df = qfq_df[qfq_df['date'] > 20100000]
 
                 qfq_df = qfq_df.iloc[::-1].reset_index(drop=True)
@@ -95,6 +95,7 @@ class Database:
                 t_index = pd.DatetimeIndex(pd.date_range(start=first_date, end=last_date, freq='1D'))
                 qfq_df = qfq_df.reindex(t_index)
                 qfq_df = qfq_df.fillna(method='bfill')
+                qfq_df = qfq_df.fillna(method='ffill')
                 qfq_df = qfq_df.reset_index()
                 qfq_df['index'] = qfq_df['index'].astype(str)
                 qfq_df['index'] = qfq_df['index'].str.replace('-', '')
@@ -162,7 +163,6 @@ class Database:
 
             response = requests.get(link_str)
 
-            #result = ast.literal_eval(json.loads(response.content))
             result = json.loads(json.loads(response.content).replace("'", "\""))
 
             df_list = []
@@ -225,7 +225,6 @@ class Database:
                 if year_month not in year_month_involved_list:
                     year_month_involved_list.append(year_month)
 
-            #if end_date >= int(expiry_date_dict[max(year_month_involved_list)].strftime('%Y%m%d')):
             year_month_involved_list.append((datetime.datetime.strptime(year_month_involved_list[-1], '%y%m') + relativedelta(months=1)).strftime('%y%m'))
 
             year_month_involved_list_str = json.dumps(str(year_month_involved_list))
@@ -368,7 +367,6 @@ class Database:
             df = df.sort_values(['date', 'market_cap_mil'], ascending=[True, False])
             df = df.reset_index(drop=True)
             df['date'] = pd.to_datetime(df['date'],format='%Y-%m-%d')
-            #df['record_date'] = pd.to_datetime(df['record_date'],format='%Y%m%d')
             df = df[['date','code','issued_share_mil','market_cap_mil','cumulative_market_cap_mil']]
             return df
 
@@ -408,7 +406,6 @@ class Database:
             df = df.sort_values(by='date')
             df = df.reset_index(drop=True)
             df['date'] = pd.to_datetime(df['date'],format='%Y-%m-%d')
-            #df['record_date'] = pd.to_datetime(df['record_date'],format='%Y%m%d')
             df = df[['date','code','issued_share_mil','market_cap_mil','cumulative_market_cap_mil']]
             df = df.set_index('date')
             return df
@@ -460,7 +457,6 @@ class Database:
             json_content = json_content.replace(' nan', '\" nan\"')
 
             result = ast.literal_eval(json_content)
-            #result = json.loads(json.loads(response.content).replace("'", "\""))
 
             df_list = []
             for item in result:
@@ -493,7 +489,6 @@ class Database:
         link_str = link_url + code_str + '&' + token_str + '&' + database_str
 
         response = requests.get(link_str)
-        #result = ast.literal_eval(json.loads(response.content))
         result = json.loads(json.loads(response.content).replace("'", "\""))
 
         df = pd.DataFrame(result)
@@ -623,8 +618,7 @@ class Database:
             df['date'] = pd.to_datetime(df['date'], format='%Y%m%d')
             df = df.set_index('date')
             df = df.sort_index()
-            #df = df.reset_index(drop=False)
-            
+
             for col in df.columns:
                 if 'participants_participant_' in col:
                     df = df.rename(columns={col:col.replace('participants_participant_','participants_')})
@@ -816,25 +810,39 @@ class Database:
                     err_msg += error + ','
             print(err_msg)
 
-    def get_spx_index_const(self):
+    def get_spx_index_const(self, hist=True):
 
         link_url = 'http://www.hkfdb.net/data_api?'
         token_str = 'token=' + str(self.authToken)
-        database_str = 'database=' + 'spx_index_const'
-        link_str = link_url + '&' + token_str + '&' + database_str
-        
-        response = requests.get(link_str)
-        result = ast.literal_eval(json.loads(response.content))
 
-        df = pd.DataFrame(result)
-        '''
-        df['is_active'] = df['is_active'].astype(bool) 
-        df['is_delisted'] = df['is_delisted'].astype(bool)
+        if hist == False:
+            database_str = 'database=' + 'spx_index_const'
+            link_str = link_url + '&' + token_str + '&' + database_str
 
-        for col in df.columns:
-            if col == 'start_date' or col == 'end_date':
-                df[col] = pd.to_datetime(df[col], format='%Y-%m-%d')
-        '''
+            response = requests.get(link_str)
+            result = ast.literal_eval(json.loads(response.content))
+
+            df = pd.DataFrame(result)
+
+        else:
+            database_str = 'database=' + 'spx_const_hist'
+            link_str = link_url + '&' + token_str + '&' + database_str
+
+            response = requests.get(link_str)
+            result = ast.literal_eval(json.loads(response.content))
+
+            df = pd.DataFrame(result)
+            mode = df['end_date'].mode()[0]
+            df['is_active'] = df['is_active'].astype(bool)
+            df['is_delisted'] = df['is_delisted'].astype(bool)
+
+            for i in range(len(df)):
+                end_date = df.loc[i,'end_date']
+                if end_date == mode:
+                    df.at[i,'end_date'] = datetime.datetime.now().strftime('%Y-%m-%d')
+            for col in df.columns:
+                if col == 'start_date' or col == 'end_date':
+                    df[col] = pd.to_datetime(df[col], format='%Y-%m-%d')
 
         return df
     
@@ -937,11 +945,12 @@ class Database:
         
         response = requests.get(link_str)
         r_content = response.content
-        json1 = json.loads(r_content)
         json1 = json.loads(r_content).replace("'", "\"")
         json1 = json1.replace('nan','NaN')
         json1 = json1.replace('INT\"L','INTL')
         json1 = json1.replace('L\"OCCITANE','LOCCITANE')
+        json1 = json1.replace('True','\"True\"')
+        json1 = json1.replace('False','\"False\"')
 
         result = json.loads(json1)
 
@@ -953,11 +962,12 @@ class Database:
         df['share_issued'] = df['share_issued'].str.replace('-','')
         df['share_issued'] = df['share_issued'].astype('int64')
 
-        df['market_capital'] = df['share_issued'] * df['price']
+        df = df.sort_values(by='code')
+
         df['market_capital'] = df['market_capital'].map(lambda x:round(x,0))
         df['market_capital'] = df['market_capital'].astype('int64')
 
-        df = df[['code','stock_name','price','PE','EPS','dividend','volume','turnover','share_issued','lot_size','market_capital','ipo_date','year_end_date']]
+        df = df[['code','stock_name','price','PE','EPS','dividend','volume','turnover','share_issued','lot_size','market_capital','ipo_date','year_end_date', 'is_stop_trading']]
 
         return df
 
@@ -1064,8 +1074,10 @@ class Database:
             df['date'] = pd.to_datetime(df['date'], format='%Y%m%d')
 
             df = df.set_index(keys='date')
-            #df = df.reset_index(drop=False)
             df = df.sort_index()
+            for col in df.columns:
+                df[col] = df[col].astype(int)
+
             return df
 
         else:
@@ -1121,7 +1133,6 @@ class Database:
             elif deri == 'fut':
                 df = df.sort_values(by='date')
                 df = df.set_index('date')
-                #df = df.reset_index(drop=True)
 
             if 'strike_price' in df.columns:
                 df = df.rename(columns={'strike_price': 'strike'})
